/******************************************************************************
 *
 *  PalmOrb - Emulate a Matrix Orbital LCD Display with Keypad
 *  Copyright (C) 2004  Chris Lansley  <palmorb@lansley.me.uk>
 *  Copyright (C) 2001  David Douthitt 
 *  Copyright (C) 2000  J Robert Ray
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *  $Source: /cvsroot/palmorb/palmorb/debug.cpp,v $
 *  $Revision: 1.3 $ $Date: 2004/10/02 00:20:10 $
 *
 ******************************************************************************/

    /***************************************************************/
    /* This file contains code that is only needed in debug builds.*/
    /***************************************************************/

#include <PalmOS.h>
#include "debug.h"
#include "resources.h"
#include "types.h"

#if DEBUG | DEBUG_ASSERTS
// Allocate up front - in case the error is that the stack is about to overflow.
static Char buf[50];
#endif

#if DEBUG
static MUInt8 hasGetStackInfo=0;

void
DebugInit(void)
{
	UInt32 romVersion;
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion); 

	if (romVersion >= sysMakeROMVersion(3,0,0,sysROMStageRelease,0))
	{
		hasGetStackInfo=1;
	}

}

/*
 * Check that there's loads of stack space left.
 */
void
DebugCheckStack(void)
{
	MemPtr start, end;
	UInt32 somethingOnStack;
	Int32 left;

	if (!hasGetStackInfo)
		return;

	SysGetStackInfo(&start, &end);
	/* ignore return value (overflowed) for the
	 * moment as it was buggy until PalmOS 3.5 
	 */

	// We're using somethingOnStack as an approx current position
	// of the stack.
	left = (char *)&somethingOnStack - (char*)start;
	if (left < 512)
	{  // less than 512 bytes from the end of the stack...
		StrIToA(buf, left);
		FrmCustomAlert(StackLowAlert, buf, " ", " ");
	}
}
#endif

#if DEBUG_ASSERTS

void
DebugOutput(Char *filename, int linenum, int arg1=0, int arg2=0)
{
	Char lineBuf[maxStrIToALen];
	StrIToA(lineBuf, linenum);

	StrPrintF(buf, "0x%x 0x%x", arg1, arg2);
	FrmCustomAlert(AssertAlert, filename, lineBuf, buf);

#if DEBUG_ASSERTS > 1
	StrPrintF(buf, "ASSERT: %s:%u (0x%x, 0x%x)", filename, linenum, arg1, arg2);
	ErrFatalDisplay( buf); // an alert doesn't stop gremlins...
#endif
}

#endif
