/******************************************************************************
 *
 *  PalmOrb - Emulate a Matrix Orbital LCD Display with Keypad
 *  Copyright (C) 2004  Chris Lansley  <palmorb@lansley.me.uk>
 *  Copyright (C) 2001  David Douthitt 
 *  Copyright (C) 2000  J Robert Ray
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *  $Source: /cvsroot/palmorb/palmorb/palmorb.h,v $
 *  $Revision: 1.11 $ $Date: 2004/10/15 19:01:57 $
 *
 ******************************************************************************/

#ifndef __PALMORB_H__
#define __PALMORB_H__

#include <PalmOS.h>

#include "resources.h"
#include "MatrixOrbital.h" // for EmulationState


#define CurrentPrefVersion 12

#define SERIAL_PARITY_NONE 0
#define SERIAL_PARITY_ODD 1
#define SERIAL_PARITY_EVEN 2

#define SERIAL_DEV_AUTO 1
#define SERIAL_DEV_RS232 2
#define SERIAL_DEV_USB 3
#define SERIAL_DEV_BT 4
#define SERIAL_DEV_IR 5

typedef struct ApplicationPreferences 
{
	UsedEmulationState defaults;

	UInt8 colorMode;		
	Boolean showKeyboard;
	Boolean showOutputs;
	Boolean showCustomChars;
	Boolean backlight;
	Boolean output1Beep;
	UInt8 refreshRate;
	Boolean baudrateChanges;
	Boolean correctSlash;
	Boolean padCharCells;
	UInt32 serialBaud;
	UInt8 serialParity;
	UInt8 serialStopBits;
	UInt8 serialDataBits;
	Boolean serialFlowCtl;
	Boolean warnSerialErrors;
	Boolean warnProtocolErrors;
	Boolean seenFeedBack;

	/*added in v9*/
	Boolean serialErrDetails;
	Boolean serialBigBuf;

	/*added in v10*/
	UInt8 serialDev;

	/*added in v11*/
	UInt8  sendGraffiti;

	/*added in v12*/
	UInt8 sendKeyUpDown;
};

extern ApplicationPreferences AppPrefs;

/* for ui.cpp */
extern void PalmOrb_SysInit(void) SEG2;
extern void Palmorb_SetSerialDefaults(void) SEG2;
extern void Palmorb_SetEmuDefaults(UsedEmulationState *defaults) SEG2;

#endif			// __PALMORB_H__
