#!/usr/bin/perl -w
#******************************************************************************
#*
#*  PalmOrb - Emulate a Matrix Orbital LCD Display with Keypad
#*  Copyright (C) 2004  Chris Lansley  <palmorb@lansley.me.uk>
#*  Copyright (C) 2001  David Douthitt 
#*  Copyright (C) 2000  J Robert Ray
#*
#*  This program is free software; you can redistribute it and/or
#*  modify it under the terms of the GNU General Public License
#*  as published by the Free Software Foundation; either version 2
#*  of the License, or (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program; if not, write to the Free Software
#*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#*
#*  $Source: /cvsroot/palmorb/palmorb/tests/decode_capture.pl,v $
#*  $Revision: 1.1 $ $Date: 2004/09/07 21:03:12 $
#*
#******************************************************************************

sub mydie($)
{
	my ($msg) = @_;
	printf("\n\nEXIT: $msg\n");
	exit(-1);
}

my @cmd;
my @cmdCount;

$cmd[35] = ["displayBigNum", 2];
$cmd[38]=["GetKeyBuf", 0];
$cmd[51]=["SetI2CAddress", 1];
$cmd[52]=["SetSerialNo", 2];
$cmd[53]=["OutputSerialNo", 0];
$cmd[54]=["GetFirmwareVer", 0];
$cmd[55]=["GetModuleType", 0]; 
$cmd[57]=["SetBaud", 1];
$cmd[58]=["SetFlowControlOn", 2];
$cmd[59]=["SetFlowControlOff", 0];
$cmd[61]=["drawVBar", 2];
$cmd[64]=["SetStartupScreen", 80];
$cmd[65]=["SetAutotransmitOn", 0];
$cmd[66]=["SetBacklightOn", 1];
$cmd[67]=["SetLinewrapOn", 0];
$cmd[68]=["SetLinewrapOff", 0];
$cmd[69]=["ClearKeyBuf", 0];
$cmd[70]=["SetBacklightOff", 0];
$cmd[71]=["MoveToXY", 2];
$cmd[72]=["Home", 0];
$cmd[74]=["SetUnderlineOn", 0];
$cmd[75]=["SetUnderlineOff", 0];
$cmd[76]=["Pos--", 0];
$cmd[77]=["Pos++", 0];
$cmd[78]=["DefineChar", 9];
$cmd[79]=["SetAutotransmitOff", 0];
$cmd[80]=["SetContrast", 1];
$cmd[81]=["SetAutoscrollOn", 0];
$cmd[82]=["SetAutoscrollOff", 0];
$cmd[83]=["SetBlockOn", 0];
$cmd[84]=["SetBlockOff", 0];
$cmd[85]=["SetDebounceTime", 1];
$cmd[86]=["SetOutputOff", 1];
$cmd[87]=["SetOutputOn", 1];
$cmd[88]=["Clear", 0];
$cmd[96]=["SetAutorepeatOff", 0];
$cmd[104]=["InitHBar", 0];
$cmd[110]=["InitBigNum", 0];
$cmd[115]=["InitVBarNarrow", 0];
$cmd[118]=["InitVBarWide", 0];
$cmd[124]=["DrawHBar", 4];
$cmd[126]=["SetAutorepeatOn", 1];
$cmd[145]=["SetContrast", 1];
$cmd[147]=["Remember", 1];
# special one for stats
$cmd[999]=["Characters", 0];


my $started=0;
my $not_on_newline=0;
my $byte;
while(read(STDIN, $byte, 1))
{
	$byte = ord($byte);

	if ($byte eq 254)
	{ # Command sequence
		my $cmdbyte;

		if ($not_on_newline)
		{
			$not_on_newline=0;
			print("\n");
		}

		read(STDIN, $cmdbyte, 1) || mydie("stopped in cmd seq");
		$cmdbyte=ord($cmdbyte);


		if (!exists($cmd[$cmdbyte]))
		{
			printf("************************************\n");
			printf("UNKNOWN CMD: %02X\n", $cmdbyte);
			printf("************************************\n");
			next;
		}

		$cmdCount[$cmdbyte]=0 if (!exists($cmdCount[$cmdbyte]));
		$cmdCount[$cmdbyte]++;


		my $cmd_name = $cmd[$cmdbyte]->[0];	
		my $cmd_args = $cmd[$cmdbyte]->[1];	
		
		print $cmd_name."(";

		my $define_char="";
		while ($cmd_args)
		{
			my $arg;

			read(STDIN, $arg, 1) || mydie("stopped in cmd args");
			$arg=ord($arg);
			if (($cmdbyte == 78) && ($cmd_args<9))
			{  # Define Char
				my $bin = sprintf("%08b", $arg);
				$bin =~ s/1/#/g;
				$bin =~ s/0/ /g;
				$define_char.="\t\t\t".$bin."\n";
			}
			printf("%d", $arg);
			$cmd_args--;
			if ($cmd_args)
			{
				print ", ";
			}
		}
		print(")\n");
		if ($cmdbyte==78)
		{
			print($define_char);
		}
	}
	else
	{
		$cmdCount[999]=0 if (!exists($cmdCount[999]));
		$cmdCount[999]++;

		if ($byte == ord('\n')) { print "<LF>" }
		elsif ($byte == 8) { print "<BS>" }
		elsif ($byte == 0xc) { print "<ClearScreen>" }
        elsif ($byte == ord('\r')) { print "<CR>" }
 		elsif ($byte == 32) { print "<SP>" }
		elsif (($byte < 32) || ($byte >= 127))
		{
			 printf ("<%02x>", $byte);
		}
		else
		{
			 printf("%c", $byte);
		}

		$not_on_newline=1;
	}
}

my $i;
for ($i=0; $i<1000; $i++)
{
	if (exists($cmdCount[$i]))
	{
		printf("%s: %d\n",$cmd[$i]->[0], $cmdCount[$i]);
	}
}
 
