/******************************************************************************
 *
 *  PalmOrb - Emulate a Matrix Orbital LCD Display with Keypad
 *  Copyright (C) 2004  Chris Lansley  <palmorb@lansley.me.uk>
 *  Copyright (C) 2001  David Douthitt 
 *  Copyright (C) 2000  J Robert Ray
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *  $Source: /cvsroot/palmorb/palmorb/ui.cpp,v $
 *  $Revision: 1.4 $ $Date: 2004/10/15 19:01:57 $
 *
 ******************************************************************************/

        /*************************************************/
        /* This file contains mostly the GUI code.       */
        /*************************************************/
#include <PalmOS.h>

#include "resources.h"
#include "MatrixOrbital.h"
#include "palmorb.h"
#include "screen.h"
#include "debug.h"
#include "serial.h"
#include "segments.h"
#include "ui.h"

/*
 * Exported functions 
 */
Boolean UI_ApplicationHandleEvent(EventPtr event);
void UI_showPowerStatus(FormType *formPtr, Boolean lowBattery) SEG2;

/* Exported variables */
Boolean UI_mainOnScreen = 0;
Boolean UI_mainDrawingOK = 0;

/*
 * local functions 
 */

// General Utility Functions
static void displayNumber(FormType *frm, UInt32 num, UInt16 objID)	SEG2;
static void displayString(FormType *frm, Char *s, UInt16 objID)	SEG2;
static void setCheckbox(FormType *frm, UInt16 objID, UInt8 set)	SEG2;
static Boolean getCheckbox(FormType *frm, UInt16 objID)	SEG2;
static Boolean oKCtlHandler(EventPtr event)	SEG2;
// Form: Main Form handling code
static Boolean mainFormHandleEvent(EventPtr event);
static Boolean buttonHandler(EventPtr event) SEG2;
// Menu handling code
static Boolean myMenuHandleEvent(EventPtr event) SEG2;
// Form: Menu->Options->Screen... handling code
static Boolean screenFormHandleEvent(EventPtr event) SEG2;
static void setScreenStates(FormType *frm) SEG2;
static Boolean screenCtlHandler(FormType *frm, EventPtr event) SEG2;
// Form: Menu->Options->Serial... handling code
static Boolean settingsSerialFormHandleEvent(EventPtr event) SEG2;
static void setSettingsSerialStates(FormType *formPtr) SEG2;
static Boolean settingsSerialCtlHandler(FormType *frm, EventPtr event) SEG2;
// Form: Menu->Options->Misc... handling code
static Boolean settingsOtherFormHandleEvent(EventPtr event)	SEG2;
static void setSettingsOtherStates(FormType *formPtr) SEG2;
static Boolean settingsOtherCtlHandler(FormType *frm, EventPtr event) SEG2;
// Form: Menu->Options->About... handling code
static Boolean aboutFormHandleEvent(EventPtr event)	SEG2;
// Form: Menu->Emulation->Current State... handling code
static Boolean stateFormHandleEvent(EventPtr event)	SEG2;
static void displayStateValues(FormType *frm) SEG2;
static Boolean stateCtlHandler(FormType *frm, EventPtr event) SEG2;
static void updateMOState(FormType *frm) SEG2;
// Form: Menu->Emulation->Ignored State... handling code
static Boolean extendedFormHandleEvent(EventPtr event) SEG2;
static void displayExtendedValues(FormType *frm) SEG2;
// Form: Menu->Emulation->Default State... handling code
static Boolean defaultFormHandleEvent(EventPtr event) SEG2;
static void displayDefaultValues(FormType *frm)	SEG2;
static Boolean defaultCtlHandler(FormType *frm, EventPtr event)	SEG2;
static void updateDefState(FormType *formPtr) SEG2;
// Form: Menu->Debug->Serial Errors... handling code
static Boolean serialErrorsFormHandleEvent(EventPtr event) SEG2;
static void displaySerialErrorValues(FormType *frm)	SEG2;
// Form: Menu->Debug->Serial Stats... handling code
static Boolean debugFormHandleEvent(EventPtr event)	SEG2;
static void displayDebugValues(FormType *frm) SEG2;
// Form: Menu->Debug->Show Font Map... handling code
static Boolean fontFormHandleEvent(EventPtr event)	SEG2;


/*****************************************************************************
 *            General Utility Functions
 ****************************************************************************/

static void
displayNumber(FormType *frm, UInt32 num, UInt16 objID)	//SEG2
{
	UInt16 objIndex;
	Char buf[maxStrIToALen];

	ASSERT(frm, "frm is NULL!");

	objIndex = FrmGetObjectIndex(frm, objID);
	ASSERT1(objIndex != frmInvalidObjectId, "objID is invalid object id",
			objID);

	if (objIndex == frmInvalidObjectId)
		return;

	StrIToA(buf, num);
	FrmHideObject(frm, objIndex);
	FrmCopyLabel(frm, objID, buf);
	FrmShowObject(frm, objIndex);
}

static void
displayString(FormType *frm, Char *s, UInt16 objID)	//SEG2
{
	UInt32 objIndex;

	ASSERT(frm, "frm is NULL!");

	objIndex = FrmGetObjectIndex(frm, objID);
	ASSERT(objIndex != frmInvalidObjectId, "objID is invalid object id");

	if (objIndex == frmInvalidObjectId)
		return;

	FrmHideObject(frm, objIndex);
	FrmCopyLabel(frm, objID, s);
	FrmShowObject(frm, objIndex);
}

static void
setCheckbox(FormType *frm, UInt16 objID, UInt8 set)	//SEG2
{
	ControlType *controlType;
	UInt16 objIndex;

	ASSERT(frm, "frm is NULL");

	objIndex = FrmGetObjectIndex(frm, objID);
	ASSERT(objIndex != frmInvalidObjectId, "objID is invalid object id");

	if (objIndex == frmInvalidObjectId)
		return;

	controlType = (ControlType *) FrmGetObjectPtr(frm, objIndex);
	ASSERT(controlType, "controlType is NULL");

	if (controlType)
		CtlSetValue(controlType, (set == 0) ? 0 : 1);
}

static Boolean
getCheckbox(FormType *frm, UInt16 objID)	//SEG2
{
	ControlType *controlType;
	UInt16 objIndex;

	ASSERT(frm, "frm is NULL");

	objIndex = FrmGetObjectIndex(frm, objID);
	ASSERT1(objIndex != frmInvalidObjectId, "objID is invalid object id",
			objID);

	if (objIndex != frmInvalidObjectId)
	{
		controlType = (ControlType *) FrmGetObjectPtr(frm, objIndex);
		ASSERT(controlType, "controlType is NULL");

		if (controlType)
			return CtlGetValue(controlType);
	}

	return 0;
}

/*
 * Simple General Ctl Handler that just returns to MainFormID when
 * OK is pressed.
 */
static Boolean
oKCtlHandler(EventPtr event)	//SEG2
{
	Boolean handled = false;

	switch (event->data.ctlSelect.controlID)
	{

		case OkButton:
			FrmReturnToForm(MainFormID);
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}

/*****************************************************************************
 *            Battery low symbol
 ****************************************************************************/

void
UI_showPowerStatus(FormType *formPtr, Boolean lowBattery)	//SEG2
{
	if (formPtr==NULL)
		return;

	if (UI_mainOnScreen && UI_mainDrawingOK)
	{
		if (lowBattery)
			FrmShowObject(formPtr, FrmGetObjectIndex(formPtr, PowerBitmap));
		else
			FrmHideObject(formPtr, FrmGetObjectIndex(formPtr, PowerBitmap));
	}
}


/*****************************************************************************
 *            Form handling - main entry point.
 ****************************************************************************/

Boolean
UI_ApplicationHandleEvent(EventPtr event)
{
	Boolean handled = false;

	if (event->eType == frmLoadEvent)
	{
		UInt16 frmId = event->data.frmLoad.formID;
		FormPtr frm = FrmInitForm(frmId);
		ASSERT(frm, "frm is NULL");

		FrmSetActiveForm(frm);

		switch (frmId)
		{
			case MainFormID:
				FrmSetEventHandler(frm, mainFormHandleEvent);
				//FrmSetMenu(frm, MainMenuID); // set in resources
				handled = true;
				break;

			case StateFormID:
				FrmSetEventHandler(frm, stateFormHandleEvent);
				handled = true;
				break;

			case DefaultsFormID:
				FrmSetEventHandler(frm, defaultFormHandleEvent);
				handled = true;
				break;

			case ExtFormID:
				FrmSetEventHandler(frm, extendedFormHandleEvent);
				handled = true;
				break;

			case AboutFormID:
				FrmSetEventHandler(frm, aboutFormHandleEvent);
				handled = true;
				break;

			case DebugFormID:
				FrmSetEventHandler(frm, debugFormHandleEvent);
				handled = true;
				break;

			case SerialErrorsFormID:
				FrmSetEventHandler(frm, serialErrorsFormHandleEvent);
				handled = true;
				break;

			case FontFormID:
				FrmSetEventHandler(frm, fontFormHandleEvent);
				handled = true;
				break;

			case ScreenFormID:
				FrmSetEventHandler(frm, screenFormHandleEvent);
				handled = true;
				break;

			case SettingsSerialFormID:
				FrmSetEventHandler(frm, settingsSerialFormHandleEvent);
				handled = true;
				break;

			case SettingsOtherFormID:
				FrmSetEventHandler(frm, settingsOtherFormHandleEvent);
				handled = true;
				break;

			default:
				// If this fires - then check out the code above the switch
				ASSERT(0, "unhandled form!");
		}
	}
	return handled;
}


/*****************************************************************************
 *            Form: Main Form handling code
 ****************************************************************************/

static Boolean
mainFormHandleEvent(EventPtr event)
{
	Boolean handled = false;
	FormType *frm = FrmGetFormPtr(MainFormID);
	ASSERT(frm, "frm is NULL!");
	if (!frm)
		return false;

	switch (event->eType)
	{
		case frmUpdateEvent:
			ASSERT1(event->data.frmUpdate.formID == MainFormID,
				   "not MainFormID: %x", event->data.frmUpdate.formID);
		case frmOpenEvent:
#if DEBUG_ASSERTS
			if (event->eType == frmOpenEvent)
				ASSERT(event->data.frmOpen.formID == MainFormID,
					   "not MainFormID");
#endif // DEBUG_ASSERTS


			FrmCopyTitle(frm, "PalmOrb: LK204-25");
			FrmDrawForm(frm);
			UI_mainOnScreen = 1;
			UI_mainDrawingOK = 1;

			Screen_SetColorMode(AppPrefs.colorMode);

			//UI_showPowerStatus(frm);
			Screen_Start();	// This will cause a redraw of the lcd.

			// Show feedback alert if not already done.
			if (!AppPrefs.seenFeedBack)
			{
				AppPrefs.seenFeedBack = 1;
				FrmAlert(FeedBackAlert);
			}
			handled = true;
			break;

		case frmCloseEvent:
			UI_mainOnScreen = 0;
			UI_mainDrawingOK = 0;
			Screen_Stop();
			break;

		case menuEvent:
			handled = myMenuHandleEvent(event);
			break;

		case ctlRepeatEvent:
			handled = buttonHandler(event);
			break;

		case nilEvent:
			handled = true;
			break;

		default:
			handled = false;
	}

	// Update the display every 0.5 seconds
	static UInt32 lasttime = 0;
	UInt32 now = TimGetTicks();
	static UInt16 ticksps = SysTicksPerSecond();
	UInt16 tick_wait = ticksps / AppPrefs.refreshRate;
	if ((lasttime + tick_wait < now) || (now < lasttime))
	{
		Screen_UpdateLcd();
		lasttime = now;
	}

	return handled;
}

static Boolean
buttonHandler(EventPtr event)	//SEG2
{
	Boolean handled = false;
	Char c = 'x';

	switch (event->data.ctlSelect.controlID)
	{
		case ButtonA: c = 'A'; handled = true; break;
		case ButtonB: c = 'B'; handled = true; break;
		case ButtonC: c = 'C'; handled = true; break;
		case ButtonD: c = 'D'; handled = true; break;
		case ButtonE: c = 'E'; handled = true; break;
		case ButtonF: c = 'F'; handled = true; break;
		case ButtonG: c = 'G'; handled = true; break;
		case ButtonH: c = 'H'; handled = true; break;
		case ButtonI: c = 'I'; handled = true; break;
		case ButtonJ: c = 'J'; handled = true; break;
		case ButtonK: c = 'K'; handled = true; break;
		case ButtonL: c = 'L'; handled = true; break;
		case ButtonM: c = 'M'; handled = true; break;
		case ButtonN: c = 'N'; handled = true; break;
		case ButtonO: c = 'O'; handled = true; break;
		case ButtonP: c = 'P'; handled = true; break;
		case ButtonQ: c = 'Q'; handled = true; break;
		case ButtonR: c = 'R'; handled = true; break;
		case ButtonS: c = 'S'; handled = true; break;
		case ButtonT: c = 'T'; handled = true; break;
		case ButtonU: c = 'U'; handled = true; break;
		case ButtonV: c = 'V'; handled = true; break;
		case ButtonW: c = 'W'; handled = true; break;
		case ButtonX: c = 'X'; handled = true; break;
		case ButtonY: c = 'Y'; handled = true; break;
		//default: do nothing... not handled...
	}

	if (handled == true)
		MO_SendKey(c);

	return false;
}



/*****************************************************************************
 *            Menu handling code
 ****************************************************************************/
static Boolean
myMenuHandleEvent(EventPtr event)	//SEG2
{
	Boolean handled = false;

	switch (event->data.menu.itemID)
	{
		case menuFontID:
			FrmPopupForm(FontFormID);
			handled = true;
			break;

		case menuExtID:
			FrmPopupForm(ExtFormID);
			handled = true;
			break;

		case menuAboutID:
			FrmPopupForm(AboutFormID);
			handled = true;
			break;

		case menuHelpID:
			FrmHelp(MainHelpID);
			handled = true;
			break;

		case menuReinitializeScreenID:
			PalmOrb_SysInit();
			FrmAlert(InitAlert);
			handled = true;
			break;

		case menuClearScreenID:
			MO_ClearScreen();
			handled = true;
			break;

		case menuScreenID:
			FrmPopupForm(ScreenFormID);
			handled = true;
			break;

		case menuSettingsSerialID:
			FrmPopupForm(SettingsSerialFormID);
			handled = true;
			break;

		case menuSettingsOtherID:
			FrmPopupForm(SettingsOtherFormID);
			handled = true;
			break;

		case menuResetOutputsID:
			MO_ClearOutputs();
			handled = true;
			break;

		case menuStateID:
			FrmPopupForm(StateFormID);
			handled = true;
			break;

		case menuDefaultID:
			FrmPopupForm(DefaultsFormID);
			handled = true;
			break;

		case menuHexDataID:
			MO_enterHexMode();
			handled = true;
			break;

		case menuParsedDataID:
			MO_enterParsedMode();
			handled = true;
			break;

		case menuNoDebugID:
			MO_returnToNormalMode();
			handled = true;
			break;

		case menuDebugID:
			FrmPopupForm(DebugFormID);
			handled = true;
			break;

		case menuSerialErrorsID:
			if (AppPrefs.serialErrDetails)
			{
				FrmPopupForm(SerialErrorsFormID);
			}
			else
			{
				FrmAlert(NoDetailedSerialErrorsAlert);
			}
			handled = true;
			break;

		default:
			handled = false;
			break;
	}

#if 0
	if (handled == true)
	{
		//CJL: Look at this
		FormType *formPtr = FrmGetFormPtr(MainFormID);
		ASSERT(formPtr, "MainForm not loaded");
		FrmSetMenu(formPtr, MainMenuID);
	}
#endif

	return handled;
}

/*****************************************************************************
 *            Form: Menu->Options->Screen... handling code
 ****************************************************************************/

static Boolean
screenFormHandleEvent(EventPtr event)	//SEG2
{
	Boolean handled = false;
	FormType *frm = FrmGetFormPtr(ScreenFormID);
	ASSERT(frm, "frm is NULL!");
	if (!frm)
		return false;

	switch (event->eType)
	{
		case frmOpenEvent:
			setScreenStates(frm);
			FrmDrawForm(frm);
			handled = true;
			break;

		case ctlSelectEvent:
			handled = screenCtlHandler(frm, event);
			break;

		case nilEvent:
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}

static void
setScreenStates(FormType *frm)	//SEG2
{
	/*
	 * Color mode
	 */
	{
		if (!Screen_ColorPossible)
			AppPrefs.colorMode = COLORMODE_NONE;

		UInt16 cl_objIndex = FrmGetObjectIndex(frm, ColorListID);
		ASSERT(cl_objIndex != frmInvalidObjectId, "objID is invalid object id");

		if (cl_objIndex == frmInvalidObjectId)
			return;

		UInt16 cp_objIndex = FrmGetObjectIndex(frm, ColorPopID);
		ASSERT(cp_objIndex != frmInvalidObjectId, "objID is invalid object id");

		if (cp_objIndex == frmInvalidObjectId)
			return;

		ListType *list = (ListType *) FrmGetObjectPtr(frm, cl_objIndex);
		ASSERT(list, "list is NULL");

		ControlType *pop = (ControlType *) FrmGetObjectPtr(frm, cp_objIndex);
		ASSERT(pop, "pop is NULL");

		if (!list || !pop)
			return;

		Char *txt;

		LstSetSelection(list, AppPrefs.colorMode);
		txt = LstGetSelectionText(list, AppPrefs.colorMode);
		ASSERT(txt, "txt is NULL");
		if (txt)
			CtlSetLabel(pop, txt);

		if (!Screen_ColorPossible)
		{	// disable the list popup
			CtlSetEnabled(pop, false);
		}
	}

	/*
	 * Refresh Rate
	 */
	{
		UInt16 cl_objIndex = FrmGetObjectIndex(frm, RefreshRateListID);
		ASSERT(cl_objIndex != frmInvalidObjectId, "objID is invalid object id");

		UInt16 cp_objIndex = FrmGetObjectIndex(frm, RefreshRatePopID);
		ASSERT(cp_objIndex != frmInvalidObjectId, "objID is invalid object id");

		if ((cl_objIndex == frmInvalidObjectId) ||
			(cp_objIndex == frmInvalidObjectId))
			return;

		ListType *list = (ListType *) FrmGetObjectPtr(frm, cl_objIndex);
		ASSERT(list, "list is NULL");
		ControlType *pop = (ControlType *) FrmGetObjectPtr(frm, cp_objIndex);
		ASSERT(pop, "pop is NULL");


		if (!list || !pop)
			return;

		Char *txt;

		LstSetSelection(list, AppPrefs.refreshRate - 1);
		txt = LstGetSelectionText(list, AppPrefs.refreshRate - 1);
		ASSERT(txt, "txt is NULL");
		if (txt)
			CtlSetLabel(pop, txt);
		LstMakeItemVisible(list, AppPrefs.refreshRate - 1);
	}

	/*
	 * Remaining check boxes
	 */
	setCheckbox(frm, ShowKeyboardChkbox, AppPrefs.showKeyboard);
	setCheckbox(frm, ShowOutputsChkbox, AppPrefs.showOutputs);
	setCheckbox(frm, ShowCustomCharsChkbox, AppPrefs.showCustomChars);
	setCheckbox(frm, ShowPaddingChkbox, AppPrefs.padCharCells);
}

static Boolean
screenCtlHandler(FormType *frm, EventPtr event)	//SEG2
{
	Boolean handled = false;

	switch (event->data.ctlSelect.controlID)
	{

		case ScreenOkButton:
			{
				/*
				 * Color mode
				 */
				{
					UInt32 objIndex = FrmGetObjectIndex(frm, ColorListID);
					ASSERT(objIndex != frmInvalidObjectId,
						   "objID is invalid object id");
					if (objIndex != frmInvalidObjectId)
					{
						ListType *list;
						UInt16 selection;

						list = (ListType *) FrmGetObjectPtr(frm, objIndex);
						ASSERT(list, "list is null");

						if (list)
						{
							selection = LstGetSelection(list);

							if ((selection >= 0) && (selection <= COLORMODE_MAX)
								&& (AppPrefs.colorMode != selection))
							{
								AppPrefs.colorMode = selection;
								//Screen_SetColorMode(selection);
							}
						}

					}
				}

				/*
				 * Refresh rate
				 */
				{
					UInt32 objIndex = FrmGetObjectIndex(frm, RefreshRateListID);
					ASSERT(objIndex != frmInvalidObjectId,
						   "objID is invalid object id");
					if (objIndex != frmInvalidObjectId)
					{
						ListType *list;

						list = (ListType *) FrmGetObjectPtr(frm, objIndex);
						ASSERT(list, "list is null");
						if (list)
							AppPrefs.refreshRate = LstGetSelection(list) + 1;
					}
				}


				/*
				 * Remaining check boxes.
				 */
				AppPrefs.showKeyboard = getCheckbox(frm, ShowKeyboardChkbox);
				AppPrefs.showOutputs = getCheckbox(frm, ShowOutputsChkbox);
				AppPrefs.showCustomChars = getCheckbox(frm, ShowCustomCharsChkbox);
				AppPrefs.padCharCells = getCheckbox(frm, ShowPaddingChkbox);

				MO_PrefsChanged();
			}

			FrmReturnToForm(MainFormID);
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}

/*****************************************************************************
 *            Form: Menu->Options->Serial... handling code
 ****************************************************************************/

static Boolean
settingsSerialFormHandleEvent(EventPtr event)	//SEG2
{
	Boolean handled = false;
	FormType *frm = FrmGetFormPtr(SettingsSerialFormID);
	ASSERT(frm, "frm is NULL!");
	if (!frm)
		return false;

	switch (event->eType)
	{
		case frmOpenEvent:

			setSettingsSerialStates(frm);
			FrmDrawForm(frm);
			handled = true;
			break;

		case ctlSelectEvent:
			handled = settingsSerialCtlHandler(frm, event);
			break;

		case nilEvent:
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}

static void
setSettingsSerialStates(FormType *formPtr)	//SEG2
{
	/* 
	 * Device selection
	 */
	UInt16 dl_objIndex = FrmGetObjectIndex(formPtr, SerialDevListID);
	ASSERT(dl_objIndex != frmInvalidObjectId, "objID is invalid object id");

	UInt16 dp_objIndex = FrmGetObjectIndex(formPtr, SerialDevPopID);
	ASSERT(dp_objIndex != frmInvalidObjectId, "objID is invalid object id");

	if ((dl_objIndex == frmInvalidObjectId) ||
		(dp_objIndex == frmInvalidObjectId))
		return;

	ListType *dlist = (ListType *) FrmGetObjectPtr(formPtr, dl_objIndex);
	ASSERT(dlist, "list is NULL");
	ControlType *dpop = (ControlType *) FrmGetObjectPtr(formPtr, dp_objIndex);
	ASSERT(dpop, "dpop is NULL");

	if (!dlist || !dpop)
		return;

	UInt8 doption;
	if (Serial_newMan)
	{
		// new serial manager supported - so display full list
		doption = AppPrefs.serialDev - 1;
	}
	else
	{   // only serial available
		doption = SERIAL_DEV_RS232 - 1;
	}

	Char *dtxt;

	LstSetSelection(dlist, doption);
	dtxt = LstGetSelectionText(dlist, doption);
	ASSERT(dtxt, "dtxt is NULL");
	if (dtxt)
		CtlSetLabel(dpop, dtxt);
	LstMakeItemVisible(dlist, doption);

	if (!Serial_newMan)
	{
		// only one option so disable list
		CtlSetEnabled(dpop, false);
	}

	/*
	 * BAUD RATE 
	 */ 
	UInt16 cl_objIndex = FrmGetObjectIndex(formPtr, SerialBaudListID);
	ASSERT(cl_objIndex != frmInvalidObjectId, "objID is invalid object id");

	UInt16 cp_objIndex = FrmGetObjectIndex(formPtr, SerialBaudPopID);
	ASSERT(cp_objIndex != frmInvalidObjectId, "objID is invalid object id");

	if ((cl_objIndex == frmInvalidObjectId) ||
		(cp_objIndex == frmInvalidObjectId))
		return;

	ListType *list = (ListType *) FrmGetObjectPtr(formPtr, cl_objIndex);
	ASSERT(list, "list is NULL");
	ControlType *pop = (ControlType *) FrmGetObjectPtr(formPtr, cp_objIndex);
	ASSERT(pop, "pop is NULL");


	if (!list || !pop)
		return;

	UInt8 option;
	switch (AppPrefs.serialBaud)
	{
		case 1200: option = 0; break;
		case 2400: option = 1; break;
		case 4800: option = 2; break;
		case 9600: option = 3; break;
		case 19200: option = 4; break;
		case 38400: option = 5; break;
		case 57600: option = 6; break;
		case 115200: option = 7; break;
		default:
			ASSERT(0, "unknown baud rate");
			AppPrefs.serialBaud = 19200;
			option = 4;
			break;
	}

	Char *txt;

	LstSetSelection(list, option);
	txt = LstGetSelectionText(list, option);
	ASSERT(txt, "txt is NULL");
	if (txt)
		CtlSetLabel(pop, txt);
	LstMakeItemVisible(list, option);

	/*
	 * Data bits
	 */

	UInt16 select = SerialBits5;
	if (AppPrefs.serialDataBits == 5)
		select = SerialBits5;
	else if (AppPrefs.serialDataBits == 6)
		select = SerialBits6;
	else if (AppPrefs.serialDataBits == 7)
		select = SerialBits7;
	else if (AppPrefs.serialDataBits == 8)
		select = SerialBits8;
	else
		ASSERT(0, "serialDataBits is invalid");
	FrmSetControlGroupSelection(formPtr, SerialBits, select);


	/*
	 * Data parity
	 */
	select = SerialParityOdd;
	if (AppPrefs.serialParity == SERIAL_PARITY_NONE)
		select = SerialParityNone;
	else if (AppPrefs.serialParity == SERIAL_PARITY_ODD)
		select = SerialParityOdd;
	else if (AppPrefs.serialParity == SERIAL_PARITY_EVEN)
		select = SerialParityEven;
	else
		ASSERT(0, "serialParity is invalid");
	FrmSetControlGroupSelection(formPtr, SerialParity, select);

	/*
	 * Flow control
	 */
	select = SerialFlowOn;
	if (AppPrefs.serialFlowCtl == 0)
		select = SerialFlowOff;
	else if (AppPrefs.serialFlowCtl == 1)
		select = SerialFlowOn;
	else
		ASSERT(0, "serialFlowCtl is invalid");
	FrmSetControlGroupSelection(formPtr, SerialFlow, select);

	/*
	 * Stop bits
	 */
	select = SerialStop1;
	if (AppPrefs.serialStopBits == 1)
		select = SerialStop1;
	else if (AppPrefs.serialStopBits == 2)
		select = SerialStop2;
	else
		ASSERT(0, "serialStopBits is invalid");
	FrmSetControlGroupSelection(formPtr, SerialStop, select);

	/*
	 * Remaining check boxes.
	 */
	setCheckbox(formPtr, SerialBigBufChkbox, AppPrefs.serialBigBuf);
	setCheckbox(formPtr, SerialDetailedErrsChkbox, AppPrefs.serialErrDetails);
}

static Boolean
settingsSerialCtlHandler(FormType *frm, EventPtr event)	//SEG2
{
	Boolean handled = false;

	switch (event->data.ctlSelect.controlID)
	{

		case SettingsSerialOkButton:
			{
				/* 
				 * Device selection
				 */
				UInt32 dobjIndex = FrmGetObjectIndex(frm, SerialDevListID);
				ASSERT(dobjIndex != frmInvalidObjectId,
					   "objID is invalid object id");
				if (dobjIndex != frmInvalidObjectId)
				{
					ListType *list;

					list = (ListType *) FrmGetObjectPtr(frm, dobjIndex);
					ASSERT(list, "list is null");
					if (list)
						AppPrefs.serialDev = LstGetSelection(list) + 1;
				}
				/* 
				 * Baud rate 
				 */
				UInt32 objIndex = FrmGetObjectIndex(frm, SerialBaudListID);
				ASSERT(objIndex != frmInvalidObjectId,
					   "objID is invalid object id");
				if (objIndex != frmInvalidObjectId)
				{
					ListType *list;

					list = (ListType *) FrmGetObjectPtr(frm, objIndex);
					ASSERT(list, "list is null");
					if (list)
					{
						Char *txt =
							LstGetSelectionText(list, LstGetSelection(list));
						if (txt)
							AppPrefs.serialBaud = StrAToI(txt);
					}
				}

				/*
				 * Data bits 
				 */
				UInt16 bitsObjIndex = FrmGetControlGroupSelection(frm, SerialBits);
				ASSERT(bitsObjIndex != frmNoSelectedControl, "none selected");
				UInt16 bitsObjId = FrmGetObjectId(frm, bitsObjIndex);
				ASSERT(bitsObjId != frmInvalidObjectId, "invalid objId");

				if (bitsObjId == SerialBits5)
					AppPrefs.serialDataBits = 5;
				else if (bitsObjId == SerialBits6)
					AppPrefs.serialDataBits = 6;
				else if (bitsObjId == SerialBits7)
					AppPrefs.serialDataBits = 7;
				else if (bitsObjId == SerialBits8)
					AppPrefs.serialDataBits = 8;
				else
					ASSERT(0, "invalid data bits selection");

				/*
				 * Parity
				 */
				UInt16 pariObjIndex = FrmGetControlGroupSelection(frm, SerialParity);
				ASSERT(pariObjIndex != frmNoSelectedControl, "none selected");
				UInt16 pariObjId = FrmGetObjectId(frm, pariObjIndex);
				ASSERT(pariObjId != frmInvalidObjectId, "invalid objId");

				switch (pariObjId)
				{
					case SerialParityNone:
						AppPrefs.serialParity = SERIAL_PARITY_NONE;
						break;
					case SerialParityOdd:
						AppPrefs.serialParity = SERIAL_PARITY_ODD;
						break;
					case SerialParityEven:
						AppPrefs.serialParity = SERIAL_PARITY_EVEN;
						break;
					default:
						ASSERT(0, "invalid parity selection");
				}

				/*
				 * Stop bits
				 */
				UInt16 stopObjIndex = FrmGetControlGroupSelection(frm, SerialStop);
				ASSERT(stopObjIndex != frmNoSelectedControl, "none selected");
				UInt16 stopObjId = FrmGetObjectId(frm, stopObjIndex);
				ASSERT(stopObjId != frmInvalidObjectId, "invalid objId");

				if (stopObjId == SerialStop1)
					AppPrefs.serialStopBits = 1;
				else if (stopObjId == SerialStop2)
					AppPrefs.serialStopBits = 2;
				else
					ASSERT(0, "invalid stop bit selection");

				/*
				 * Flow control
				 */
				UInt16 flowObjIndex = FrmGetControlGroupSelection(frm, SerialFlow);
				ASSERT(flowObjIndex != frmNoSelectedControl, "none selected");
				UInt16 flowObjId = FrmGetObjectId(frm, flowObjIndex);
				ASSERT(flowObjId != frmInvalidObjectId, "invalid objId");

				if (flowObjId == SerialFlowOn)
					AppPrefs.serialFlowCtl = 1;
				else if (flowObjId == SerialFlowOff)
					AppPrefs.serialFlowCtl = 0;
				else
					ASSERT(0, "invalid flow selection");

				/*
				 * Remaining check boxes
				 */
				AppPrefs.serialErrDetails = getCheckbox(frm, SerialDetailedErrsChkbox);
				AppPrefs.serialBigBuf = getCheckbox(frm, SerialBigBufChkbox);

			}
			
			// Update settings by closing/opening port.
			Serial_ClosePort();
			if (Serial_OpenPort() == errNone)
				FrmReturnToForm(MainFormID);
			//else make them change the settings

			handled = true;
			break;

		case SettingsSerialCancelButton:
			FrmReturnToForm(MainFormID);
			handled = true;
			break;

		case SettingsSerialDefaultsButton:
			Palmorb_SetSerialDefaults();
			setSettingsSerialStates(frm);
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}

/*****************************************************************************
 *            Form: Menu->Options->Misc... handling code
 ****************************************************************************/

static Boolean
settingsOtherFormHandleEvent(EventPtr event)	//SEG2
{
	Boolean handled = false;
	FormType *frm = FrmGetFormPtr(SettingsOtherFormID);
	ASSERT(frm, "frm is NULL!");
	if (!frm)
		return false;

	switch (event->eType)
	{
		case frmOpenEvent:
			setSettingsOtherStates(frm);
			FrmDrawForm(frm);
			handled = true;
			break;

		case ctlSelectEvent:
			handled = settingsOtherCtlHandler(frm, event);
			break;

		case nilEvent:
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}

static void
setSettingsOtherStates(FormType *formPtr)	//SEG2
{

	setCheckbox(formPtr, Output1IsBeepChkbox, AppPrefs.output1Beep);
	setCheckbox(formPtr, AllowBacklightChkbox, AppPrefs.backlight);
	setCheckbox(formPtr, CorrectSlashChkbox, AppPrefs.correctSlash);
	setCheckbox(formPtr, BaudRateChangesChkbox, AppPrefs.baudrateChanges);
	setCheckbox(formPtr, WarnSerialErrorsChkbox, AppPrefs.warnSerialErrors);
	setCheckbox(formPtr, WarnProtocolErrorsChkbox, AppPrefs.warnProtocolErrors);
	setCheckbox(formPtr, GraffitiChkbox, AppPrefs.sendGraffiti);
	setCheckbox(formPtr, KeyUpDownChkbox, AppPrefs.sendKeyUpDown);
}

static Boolean
settingsOtherCtlHandler(FormType *frm, EventPtr event)	//SEG2
{
	Boolean handled = false;

	switch (event->data.ctlSelect.controlID)
	{
		case SettingsOtherOkButton:
			{
				AppPrefs.output1Beep = getCheckbox(frm, Output1IsBeepChkbox);
				AppPrefs.backlight = getCheckbox(frm, AllowBacklightChkbox);
				AppPrefs.correctSlash = getCheckbox(frm, CorrectSlashChkbox);
				AppPrefs.baudrateChanges = getCheckbox(frm, BaudRateChangesChkbox);
				AppPrefs.warnSerialErrors = getCheckbox(frm, WarnSerialErrorsChkbox);
				AppPrefs.warnProtocolErrors = getCheckbox(frm, WarnProtocolErrorsChkbox);
				AppPrefs.sendGraffiti = getCheckbox(frm, GraffitiChkbox);
				AppPrefs.sendKeyUpDown = getCheckbox(frm, KeyUpDownChkbox);

				MO_PrefsChanged();
				Screen_PrefsChanged();
			}

			FrmReturnToForm(MainFormID);
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}

/*****************************************************************************
 *            Form: Menu->Options->About... handling code
 ****************************************************************************/

static Boolean
aboutFormHandleEvent(EventPtr event)	//SEG2
{
	Boolean handled = false;
	FormType *frm = FrmGetFormPtr(AboutFormID);
	ASSERT(frm, "frm is NULL!");
	if (!frm)
		return false;

	switch (event->eType)
	{
		case frmOpenEvent:
			FrmDrawForm(frm);
			handled = true;
			break;

		case ctlSelectEvent:
			handled = oKCtlHandler(event);
			break;

		case nilEvent:
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}

/*****************************************************************************
 *            Form: Menu->Emulation->Current State... handling code
 ****************************************************************************/

static Boolean
stateFormHandleEvent(EventPtr event)	//SEG2
{
	Boolean handled = false;
	FormType *frm = FrmGetFormPtr(StateFormID);
	ASSERT(frm, "frm is NULL!");
	if (!frm)
		return false;

	switch (event->eType)
	{
		case frmOpenEvent:

			displayStateValues(frm);
			FrmDrawForm(frm);
			handled = true;
			break;

		case ctlSelectEvent:
			handled = stateCtlHandler(frm, event);
			break;

		case nilEvent:
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}


static void
displayStateValues(FormType *frm)	//SEG2
{
	InternalEmulationState internal;
	UsedEmulationState used;

	ASSERT(frm, "frm is NULL");

	MO_GetState(&used, NULL, &internal);

	// Beware - possible buffer overrun area!
	// xPos and yPos are both limited to less than 100. So the
	// most we can expect from %u is 2 characters each. so we have:
	// 2*2 + nul + "(, )" == 9 bytes -- we will have space for 18 just 
	// in case.
	ASSERT((internal.xPos < 100) && (internal.yPos < 100), "large coords");
	if ((internal.xPos < 100) && (internal.yPos < 100))
	{
		Char coord[18];
		StrPrintF(coord, "(%u, %u)", internal.xPos, internal.yPos);
		displayString(frm, coord, PositionLabel);
		ASSERT(StrLen(coord) <= 9, "coord is more than 9 chars");
	}

	setCheckbox(frm, LinewrapChkbox, used.linewrap);
	setCheckbox(frm, AutoscrollChkbox, used.autoscroll);
	setCheckbox(frm, AutotransmitChkbox, used.autotransmit);
	setCheckbox(frm, BacklightChkbox, used.backlight);
	setCheckbox(frm, SerialChangableChkbox, !used.serialSet);


	/*
	 * Display serial number in a field - this is ugly...
	 */
	UInt16 objIndex;
	FieldType *fld;


	objIndex = FrmGetObjectIndex(frm, SerialNumberFld);
	ASSERT(objIndex != frmInvalidObjectId, "objID is invalid object id");

	if (objIndex != frmInvalidObjectId)
	{
		fld = (FieldType *) FrmGetObjectPtr(frm, objIndex);
		ASSERT(fld, "fld is NULL");

		if (fld)
		{
			UInt16 maxSize = FldGetMaxChars(fld);
			ASSERT(maxSize, "Max Chars is 0");

			MemHandle textH = FldGetTextHandle(fld);
			if (!textH)
			{
				textH = MemHandleNew(maxSize + 1);
				ASSERT(textH, "Out of memory");
			}
			else
			{	// Manual say to remove the handle before editing the string
				FldSetTextHandle(fld, NULL);
			}

			if (textH)
			{
				Char *textP = (Char *) MemHandleLock(textH);
				ASSERT(textP, "textP is NULL");

				if (textP)
				{
					Char buf[maxStrIToALen];

					ASSERT(maxSize < maxStrIToALen, "buffer overflow");

					StrIToA(buf, used.serialNum);
					StrNCopy(textP, buf, maxSize);
					textP[maxSize] = chrNull;

					MemHandleUnlock(textH);	//FldSetTextHandle wants it unlocked.
				}
				FldSetTextHandle(fld, textH);
			}
			//FldDrawField(fld);
		}

	}
	/* END of ugly add text to field code.
	 */

	displayNumber(frm, Serial_GetBaudRate(), BaudRateLabel);
}

static Boolean
stateCtlHandler(FormType *frm, EventPtr event)	//SEG2
{
	Boolean handled = false;

	switch (event->data.ctlSelect.controlID)
	{
		case ButtonOK:
			updateMOState(frm);
			FrmReturnToForm(MainFormID);
			handled = true;
			break;

		case ButtonDefaults:
			MO_SetState(&AppPrefs.defaults);
			FrmReturnToForm(MainFormID);
			handled = true;
			break;

		case ButtonCancel:
			FrmReturnToForm(MainFormID);
			handled = true;
			break;

			//default: do nothing... not handled...
	}

	return handled;
}

static void
updateMOState(FormType *frm)	// SEG2
{
	UsedEmulationState used;

	// Get current state in case there are fields we don't know about.
	MO_GetState(&used, NULL, NULL);

	used.autoscroll = getCheckbox(frm, AutoscrollChkbox);
	used.linewrap = getCheckbox(frm, LinewrapChkbox);
	used.autotransmit = getCheckbox(frm, AutotransmitChkbox);
	used.serialSet = !getCheckbox(frm, SerialChangableChkbox);
	used.backlight = getCheckbox(frm, BacklightChkbox);

	// Get Serial Number from field
	UInt16 objIndex = FrmGetObjectIndex(frm, SerialNumberFld);
	ASSERT(objIndex != frmInvalidObjectId, "objID is invalid object id");

	if (objIndex != frmInvalidObjectId)
	{
		FieldType *fld;
		fld = (FieldType *) FrmGetObjectPtr(frm, objIndex);
		ASSERT(fld, "fld is NULL");

		if (fld)
		{
			MemHandle h = FldGetTextHandle(fld);
			ASSERT(h, "h is NULL");

			if (h)
			{
				FldSetTextHandle(fld, NULL);	// stop the fld using our memory.

				Char *p = (Char *) MemHandleLock(h);
				ASSERT(p, "p is NULL");
				if (p)
					used.serialNum = StrAToI(p);

				Err err = MemHandleUnlock(h);
				ASSERT(err != memErrInvalidParam, "MemHandleUnlock errored");

				err = MemHandleFree(h);
				ASSERT(err != memErrInvalidParam, "MemHandleFree errored");
			}
		}
	}

	MO_SetState(&used);
}

/*****************************************************************************
 *            Form: Menu->Emulation->Ignored State... handling code
 ****************************************************************************/

static Boolean
extendedFormHandleEvent(EventPtr event)	//SEG2
{
	Boolean handled = false;
	FormType *frm = FrmGetFormPtr(ExtFormID);
	ASSERT(frm, "frm is NULL!");
	if (!frm)
		return false;

	switch (event->eType)
	{
		case frmOpenEvent:
			displayExtendedValues(frm);
			FrmDrawForm(frm);
			handled = true;
			break;

		case ctlSelectEvent:
			handled = oKCtlHandler(event);
			break;

		case nilEvent:
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}

static void
displayExtendedValues(FormType *frm)	//SEG2
{
	NotUsedEmulationState nused;

	MO_GetState(NULL, &nused, NULL);

	setCheckbox(frm, UnderlineChkbox, nused.underline);
	setCheckbox(frm, BlockChkbox, nused.block);

	setCheckbox(frm, AutorepeatChkbox, nused.autorepeat);
	if (nused.autorepeatMode == 0)
		displayString(frm, "Repeat Key", AutorepeatSettingLabel);
	else if (nused.autorepeatMode == 1)
		displayString(frm, "Key Dn/Up", AutorepeatSettingLabel);
	else
		displayString(frm, "Invalid", AutorepeatSettingLabel);

	if (nused.backlightMins == 0)
		displayString(frm, "indef.", BacklightSettingLabel);
	else
		displayNumber(frm, nused.backlightMins, BacklightSettingLabel);

	displayNumber(frm, nused.contrast, ContrastSettingLabel);
	displayNumber(frm, nused.debounceTime, DebounceTimeLabel);

	// Beware - possible buffer overrun area!
	// StrItoH takes a UInt32 so thats a maximum of 8 hex characters + nil
	// We'll have space for 16 just in case.
	Char hex[16];
	StrIToH(hex, nused.addressI2C);
	displayString(frm, hex, I2CAddrLabel);
	ASSERT(StrLen(hex) <= 8, "StrIToH returned more than 8 characters");


	setCheckbox(frm, FlowControlChkbox, nused.flowControlMode);
	// Beware - possible buffer overrun area!
	// both flowControlFul and flowControlEmpty are 8 bits and so
	// we can expect a max of 3 chars for each one. So that gives
	// a max of 8 bytes - we will have space for 16 just in case.
	Char str[16];
	StrPrintF(str, "%u/%u", nused.flowControlFull, nused.flowControlEmpty);
	displayString(frm, str, FlowFullEmptyLabel);
	ASSERT(StrLen(str) < 8, "str is longer than expected");
}



/*****************************************************************************
 *            Form: Menu->Emulation->Default State... handling code
 ****************************************************************************/

static Boolean
defaultFormHandleEvent(EventPtr event)	//SEG2
{
	Boolean handled = false;
	FormType *frm = FrmGetFormPtr(DefaultsFormID);
	ASSERT(frm, "frm is NULL!");
	if (!frm)
		return false;

	switch (event->eType)
	{
		case frmOpenEvent:
			displayDefaultValues(frm);
			FrmDrawForm(frm);
			handled = true;
			break;

		case ctlSelectEvent:
			handled = defaultCtlHandler(frm, event);
			break;

		case nilEvent:
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}

static void
displayDefaultValues(FormType *frm)	//SEG2
{
	ASSERT(frm, "frm is NULL");

	setCheckbox(frm, DefLinewrapChkbox, AppPrefs.defaults.linewrap);
	setCheckbox(frm, DefAutoscrollChkbox, AppPrefs.defaults.autoscroll);
	setCheckbox(frm, DefAutotransmitChkbox, AppPrefs.defaults.autotransmit);
	setCheckbox(frm, DefSerialChangableChkbox, !AppPrefs.defaults.serialSet);
	setCheckbox(frm, DefBacklightChkbox, AppPrefs.defaults.backlight);

	/*
	 * Display serial number in a field - this is ugly...
	 */

	UInt16 objIndex = FrmGetObjectIndex(frm, DefSerialNumberFld);
	ASSERT(objIndex != frmInvalidObjectId, "objID is invalid object id");

	if (objIndex != frmInvalidObjectId)
	{
		FieldType *fld = (FieldType *) FrmGetObjectPtr(frm, objIndex);
		ASSERT(fld, "fld is NULL");

		if (fld)
		{
			UInt16 maxSize = FldGetMaxChars(fld);
			ASSERT(maxSize, "Max Chars is 0");

			MemHandle textH = FldGetTextHandle(fld);
			if (!textH)
			{
				textH = MemHandleNew(maxSize + 1);
				ASSERT(textH, "Out of memory");
			}
			else
			{	// Manual say to remove the handle before editing the string
				ASSERT(0, "why would it already have some?");
				FldSetTextHandle(fld, NULL);
			}

			if (textH)
			{
				Char *textP = (Char *) MemHandleLock(textH);
				ASSERT(textP, "textP is NULL");

				if (textP)
				{
					Char buf[maxStrIToALen];

					ASSERT(maxSize < maxStrIToALen, "buffer overflow");

					StrIToA(buf, AppPrefs.defaults.serialNum);
					StrNCopy(textP, buf, maxSize);
					textP[maxSize] = chrNull;

					MemHandleUnlock(textH);	//FldSetTextHandle wants it unlocked.
				}
				FldSetTextHandle(fld, textH);
			}
			//FldDrawField(fld);
		}
	}
	/* END of ugly add text to field code.
	 */

}

static Boolean
defaultCtlHandler(FormType *frm, EventPtr event)	//SEG2
{
	Boolean handled = false;

	switch (event->data.ctlSelect.controlID)
	{
		case DefButtonOK:
			updateDefState(frm);
			FrmReturnToForm(MainFormID);
			handled = true;
			break;

		case DefButtonDefaults:
			Palmorb_SetEmuDefaults(&AppPrefs.defaults);
			FrmReturnToForm(MainFormID);
			handled = true;
			break;

		case DefButtonCancel:
			FrmReturnToForm(MainFormID);
			handled = true;
			break;

			//default: do nothing... not handled...
	}

	return handled;
}

static void
updateDefState(FormType *formPtr)	//SEG2
{
	AppPrefs.defaults.autoscroll = getCheckbox(formPtr, DefAutoscrollChkbox);
	AppPrefs.defaults.linewrap = getCheckbox(formPtr, DefLinewrapChkbox);
	AppPrefs.defaults.autotransmit = getCheckbox(formPtr, DefAutotransmitChkbox);
	AppPrefs.defaults.serialSet = !getCheckbox(formPtr, DefSerialChangableChkbox);
	AppPrefs.defaults.backlight = getCheckbox(formPtr, DefBacklightChkbox);

	// Get Serial Number from field
	UInt16 objIndex = FrmGetObjectIndex(formPtr, DefSerialNumberFld);
	ASSERT(objIndex != frmInvalidObjectId, "objID is invalid object id");

	if (objIndex != frmInvalidObjectId)
	{
		FieldType *fld;
		fld = (FieldType *) FrmGetObjectPtr(formPtr, objIndex);
		ASSERT(fld, "fld is NULL");

		if (fld)
		{
			MemHandle h = FldGetTextHandle(fld);
			ASSERT(h, "h is NULL");

			if (h)
			{
				FldSetTextHandle(fld, NULL);	// stop the fld using our memory.

				Char *p = (Char *) MemHandleLock(h);
				ASSERT(p, "p is NULL");
				if (p)
					AppPrefs.defaults.serialNum = StrAToI(p);

				Err err = MemHandleUnlock(h);
				ASSERT(err != memErrInvalidParam, "MemHandleUnlock errored");

				err = MemHandleFree(h);
				ASSERT(err != memErrInvalidParam, "MemHandleFree errored");
			}
		}
	}
}


/*****************************************************************************
 *            Form: Menu->Debug->Serial Errors... handling code
 ****************************************************************************/

static Boolean
serialErrorsFormHandleEvent(EventPtr event)	//SEG2
{
	Boolean handled = false;
	FormType *frm = FrmGetFormPtr(SerialErrorsFormID);
	ASSERT(frm, "frm is NULL!");
	if (!frm)
		return false;

	switch (event->eType)
	{
		case frmOpenEvent:
			displaySerialErrorValues(frm);
			FrmDrawForm(frm);
			handled = true;
			break;

		case ctlSelectEvent:
			handled = oKCtlHandler(event);
			break;

		case nilEvent:
			{
				// Update the display every 0.5 seconds
				static UInt32 lasttime = 0;
				UInt32 now = TimGetTicks();
				static UInt16 tick_wait = SysTicksPerSecond() / 2;
				if ((lasttime + tick_wait < now) || (now < lasttime))
				{
					displaySerialErrorValues(frm);
					lasttime = now;
				}
			}
			handled = true;
			break;

		default:
			handled = false;
	}


	return handled;
}

static void
displaySerialErrorValues(FormType *frm)	//SEG2
{
	displayNumber(frm, Serial_LineErrorParity, SerialParityLabel);
	displayNumber(frm, Serial_LineErrorHWOverrun, SerialHWOverrunLabel);
	displayNumber(frm, Serial_LineErrorFraming, SerialFramingLabel);
	displayNumber(frm, Serial_LineErrorBreak, SerialBreakLabel);
	displayNumber(frm, Serial_LineErrorHShake, SerialHShakeLabel);
	displayNumber(frm, Serial_LineErrorHShake, SerialHShakeLabel);
	displayNumber(frm, Serial_LineErrorSWOverrun, SerialSWOverrunLabel);
	displayNumber(frm, Serial_LineErrorCarrierLost, SerialCarrierLostLabel);
	displayNumber(frm, Serial_NonLineErrors, SerialNonLineLabel);
}



/*****************************************************************************
 *            Form: Menu->Debug->Serial Stats... handling code
 ****************************************************************************/

static Boolean
debugFormHandleEvent(EventPtr event)	//SEG2
{
	Boolean handled = false;

	FormType *frm;
	frm = FrmGetFormPtr(DebugFormID);
	ASSERT(frm, "frm is NULL!");
	if (!frm)
		return false;

	switch (event->eType)
	{
		case frmOpenEvent:
			displayDebugValues(frm);
			FrmDrawForm(frm);
			handled = true;
			break;

		case ctlSelectEvent:
			handled = oKCtlHandler(event);
			break;

		case nilEvent:
			{
				// Update the display every 0.5 seconds
				static UInt32 lasttime = 0;
				UInt32 now = TimGetTicks();
				static UInt16 tick_wait = SysTicksPerSecond() / 2;
				if ((lasttime + tick_wait < now) || (now < lasttime))
				{
					displayDebugValues(frm);
					lasttime = now;
				}
			}
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}

static void
displayDebugValues(FormType *frm)	//SEG2
{
	displayNumber(frm, Serial_maxSize, MaxRecvBufLabel);
	displayNumber(frm, Serial_lastSize, LastRecvBufLabel);
	if (Serial_sizeCount > 0)
		displayNumber(frm, Serial_sizeTotal / Serial_sizeCount, AverageLabel);
	else
		displayNumber(frm, 0, AverageLabel);
	displayNumber(frm, Serial_sizeTotal, TotalRecvBufLabel);
	displayNumber(frm, Serial_sizeCount, BuffersLabel);

	displayNumber(frm, Serial_bytesPerSec, BPSLabel);
	displayNumber(frm, Serial_maxBps, MaxBPSLabel);

	displayNumber(frm, Serial_Errors, ErrorsLabel);
	displayNumber(frm, MO_ProtocolErrors, PErrorsLabel);
}


/*****************************************************************************
 *            Form: Menu->Debug->Show Font Map... handling code
 ****************************************************************************/

static Boolean
fontFormHandleEvent(EventPtr event)	//SEG2
{
	Boolean handled = false;
	FormType *frm = FrmGetFormPtr(FontFormID);
	ASSERT(frm, "frm is NULL!");
	if (!frm)
		return false;

	switch (event->eType)
	{
		case frmOpenEvent:
			{
				RectangleType rect;

				FrmDrawForm(frm);

				rect.extent.x = FW_WIDTH;
				rect.extent.y = FW_HEIGHT;

				rect.topLeft.x = 0;
				rect.topLeft.y = 0;

				WinHandle oldwin=WinSetDrawWindow(FrmGetWindowHandle(frm));

				ASSERT(Screen_Fontwin, "fontwin is NULL");
				WinCopyRectangle(Screen_Fontwin, FrmGetWindowHandle(frm),
								 &rect, 75, 25, winPaint);

				rect.topLeft.x = 75 - 1;
				rect.topLeft.y = 25 - 1;
				rect.extent.x = FW_WIDTH + 2;
				rect.extent.y = FW_HEIGHT + 2;
				WinDrawRectangleFrame(simpleFrame, &rect);

				WinSetDrawWindow(oldwin);

				handled = true;
				break;
			}

		case ctlSelectEvent:
			handled = oKCtlHandler(event);
			break;

		case nilEvent:
			handled = true;
			break;

		default:
			handled = false;
	}

	return handled;
}
